package com.example.adjust_custom_test_alone.adjust.sdk;

import android.content.Context;

import com.bun.miitmdid.core.MdidSdkHelper;
import com.bun.miitmdid.interfaces.IIdentifierListener;
import com.bun.miitmdid.interfaces.IdSupplier;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

public class Oaid_MsaSdkClient {
    public static String getOaid(Context context, final ILogger logger, long maxWaitTimeInMilli) {
        final BlockingQueue<String> oaidHolder = new LinkedBlockingQueue<>(1);

        try {
            boolean msaInternalLogging = false;
            int result = MdidSdkHelper.InitSdk(context, msaInternalLogging, new IIdentifierListener()
            {
                public void OnSupport(boolean b, IdSupplier idSupplier) {
                    try {
                        if (idSupplier == null || idSupplier.getOAID() == null) {

                            oaidHolder.offer("");
                        } else {
                            oaidHolder.offer(idSupplier.getOAID());
                        }
                    } catch (Exception e) {
                        logger.error("Fail to add %s", new Object[] { e.getMessage() });
                    }
                }
            });

            if (!isError(result, logger)) {
                return oaidHolder.poll(maxWaitTimeInMilli, TimeUnit.MILLISECONDS);
            }
        } catch (NoClassDefFoundError ex) {
            logger.error("Couldn't find msa sdk " + ex.getMessage(), new Object[0]);
        } catch (InterruptedException e) {
            logger.error("Waiting to read oaid from callback interrupted: %s", new Object[] { e
                    .getMessage() });
        } catch (Throwable t) {
            logger.error("Oaid reading process failed %s", new Object[] { t.getMessage() });
        }

        return null;
    }

    private static boolean isError(int result, ILogger logger) {
        switch (result) {
            case 1008610:
                logger.error("msa sdk error - INIT_ERROR_BEGIN", new Object[0]);
                return true;
            case 1008612:
                logger.error("msa sdk error - INIT_ERROR_DEVICE_NOSUPPORT", new Object[0]);
                return true;
            case 1008613:
                logger.error("msa sdk error - INIT_ERROR_LOAD_CONFIGFILE", new Object[0]);
                return true;
            case 1008611:
                logger.error("msa sdk error - INIT_ERROR_MANUFACTURER_NOSUPPORT", new Object[0]);
                return true;
            case 1008615:
                logger.error("msa sdk error - INIT_HELPER_CALL_ERROR", new Object[0]);
                return true;
        }
        return false;
    }
}
