
import 'package:adjust_custom_test_alone_example/second_screen.dart';
import 'package:adjust_custom_test_alone_example/shared_pref.dart';
import 'package:adjust_custom_test_alone_example/utils.dart';
import 'package:advertising_id/advertising_id.dart';
import 'package:clipboard/clipboard.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:share/share.dart';
import 'package:system_settings/system_settings.dart';

class FirstScreen extends StatefulWidget {
  const FirstScreen({Key key}) : super(key: key);

  @override
  _FirstScreenState createState() => _FirstScreenState();
}

class _FirstScreenState extends State<FirstScreen> {
  TextEditingController _baseUrlController = TextEditingController();
  TextEditingController _appTokenController = TextEditingController();

  ValueGetFuture<String> _adId;

  @override
  void initState() {
    super.initState();
    _initParameters();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: Text(WidgetsDefaultValues.pageTitle),
        ),
        body: SingleChildScrollView(
          child: Padding(
            padding: const EdgeInsets.all(10.0),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: [
                TextField(
                  textInputAction: TextInputAction.next,
                  keyboardType: TextInputType.url,
                  controller: _baseUrlController,
                  decoration: InputDecoration(
                      labelText: WidgetsDefaultValues.baseUrl,
                      suffixIcon: InkWell(
                        onTap: () {
                          _baseUrlController.clear();
                          setState(() {});
                        },
                        child: Icon(Icons.clear),
                      )),
                ),
                SizedBox(
                  height: 20.0,
                ),
                TextField(
                  textInputAction: TextInputAction.done,
                  keyboardType: TextInputType.text,
                  controller: _appTokenController,
                  decoration: InputDecoration(
                      labelText: WidgetsDefaultValues.appToken,
                      suffixIcon: InkWell(
                        onTap: () {
                          _appTokenController.clear();
                          setState(() {});
                        },
                        child: Icon(Icons.clear),
                      )),
                ),
                SizedBox(
                  height: 10.0,
                ),
                ElevatedButton(
                    onPressed: _onTapStart,
                    child: Text(WidgetsDefaultValues.start)),
                SizedBox(
                  height: 10.0,
                ),
                ElevatedButton(
                    onPressed: () async {
                      await SystemSettings.system();
                    },
                    child: Text(WidgetsDefaultValues.reset +
                        ' ' +
                        WidgetsDefaultValues.gpsAdId)),
                SizedBox(
                  height: 10.0,
                ),
                Center(child: Text(WidgetsDefaultValues.gpsAdId)),
                Container(
                  color: Colors.grey.shade300,
                  child: Padding(
                    padding: EdgeInsets.all(5),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.start,
                      children: [
                        Expanded(
                            child: Text(
                                _adId.check()?_adId.data:_adId.message )),
                        InkWell(
                          child: Icon(Icons.copy),
                          onTap: () async {
                            if (_adId.check()) {
                              await FlutterClipboard.copy(_adId.data);
                              Fluttertoast.showToast(
                                msg: 'Copied!',
                                gravity: ToastGravity.CENTER,
                              );
                            }
                          },
                        ),
                        SizedBox(
                          width: 5,
                        ),
                        InkWell(
                          child: Icon(Icons.share),
                          onTap: () async {
                            if (_adId.check()) await Share.share(_adId.data);
                          },
                        )
                      ],
                    ),
                  ),
                ),
                _urlList.check() && _tokenList.check()
                    ? ListView.builder(
                        shrinkWrap: true,
                        itemCount: _tokenList.data.length,
                        itemBuilder: (context, index) {
                          return ListTile(
                            title: Text('[${_urlList.data[index]}] '
                                '[${_tokenList.data[index]}]'),
                            onTap: () {
                              _baseUrlController.text = _urlList.data[index];
                              _appTokenController.text = _tokenList.data[index];
                              setState(() {});
                            },
                            leading: InkWell(
                                child: Icon(Icons.clear),
                                onTap: () {
                                  _urlList.data.removeAt(index);
                                  _tokenList.data.removeAt(index);
                                  setState(() {});
                                }),
                          );
                        },
                      )
                    : Container(width: 0, height: 0)
              ],
            ),
          ),
        ));
  }

  _onTapStart() async {
    if (_baseUrlController.text.isNotEmpty &&
        _appTokenController.text.isNotEmpty) {
      if((_urlList.status == Status.COMPLETED ?_urlList.data.contains(_baseUrlController.text):false)
           &&  (_tokenList.status==Status.COMPLETED?_tokenList.data.contains(_appTokenController.text):false)

          ) {

      }else{
        List<String> url = _urlList.data;
        List<String> token = _tokenList.data;
        print('check: ${_baseUrlController.text}');
        url.add(_baseUrlController.text);
        token.add(_appTokenController.text);
        print(token);
        await ShpHelper.writeStringList(
            StorageKeys.base_url_list_key, url);
        await ShpHelper.writeStringList(
            StorageKeys.app_tokens_list_key, token);
      }
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
              builder: (_) => SecondScreen(
                    baseUrl: _baseUrlController.text,
                    appToken: _appTokenController.text,
                  )));
    } else {
      Fluttertoast.showToast(msg: WidgetsDefaultValues.errorCheckInputs);
    }
  }

  ValueGetFuture<List<String>> _urlList;
  ValueGetFuture<List<String>> _tokenList;

  _initParameters() async {
    _adId = ValueGetFuture.loading(WidgetsDefaultValues.loading);
    _urlList = ValueGetFuture.loading(WidgetsDefaultValues.loading);
    _tokenList = ValueGetFuture.loading(WidgetsDefaultValues.loading);

    try {
      String adid = await AdvertisingId.id(true);
      if (adid != null)
        _adId = ValueGetFuture.completed(adid);
      else
        _adId = ValueGetFuture.error(WidgetsDefaultValues.error);
    }catch(e){
      _adId = ValueGetFuture.error(e.toString());
    }
    List<String> urls =
        await ShpHelper.readStringList(StorageKeys.base_url_list_key);
    List<String> tokens =
        await ShpHelper.readStringList(StorageKeys.app_tokens_list_key);
    print(tokens);
    if (isValueStringListOk(urls)) {
      _urlList = ValueGetFuture.completed(urls);
      _tokenList = ValueGetFuture.completed(tokens);

      _baseUrlController.text = _urlList.data.last;
      _appTokenController.text = _tokenList.data.last;
    } else {
      _urlList = ValueGetFuture.error(WidgetsDefaultValues.error);
      _tokenList = ValueGetFuture.error(WidgetsDefaultValues.error);
    }

    setState(() {});
  }
}
