
import 'package:adjust_custom_test_alone/adjust_config.dart';
import 'package:adjust_custom_test_alone/adjust_custom_test_alone.dart';
import 'package:adjust_custom_test_alone/adjust_event.dart';
import 'package:adjust_custom_test_alone/adjust_event_failure.dart';
import 'package:adjust_custom_test_alone/adjust_event_success.dart';
import 'package:adjust_custom_test_alone/adjust_session_failure.dart';
import 'package:adjust_custom_test_alone/adjust_session_success.dart';
import 'package:adjust_custom_test_alone/sdk_click.dart';
import 'package:adjust_custom_test_alone_example/first_screen.dart';
import 'package:adjust_custom_test_alone_example/shared_pref.dart';
import 'package:adjust_custom_test_alone_example/utils.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';

class SecondScreen extends StatefulWidget {
  final String appToken;
  final String baseUrl;

  SecondScreen({@required this.appToken, @required this.baseUrl});

  @override
  _SecondScreenState createState() => _SecondScreenState();
}

class _SecondScreenState extends State<SecondScreen>
    with WidgetsBindingObserver {
  TextEditingController _info1Controller = TextEditingController();
  TextEditingController _info2Controller = TextEditingController();
  TextEditingController _info3Controller = TextEditingController();
  TextEditingController _info4Controller = TextEditingController();
  TextEditingController _appSecretController = TextEditingController();
  TextEditingController _eventTokenController = TextEditingController();
  TextEditingController _eventValueController = TextEditingController();
  TextEditingController _eventRevController = TextEditingController();

  bool _isRevActive = false;
  bool _isSecretActive = false;

  double _screenWidth, _screenHeight;

  ValueGetFuture<String> _installPack = ValueGetFuture.loading('loading');
  ValueGetFuture<String> _eventPack = ValueGetFuture.loading('loading');
  ValueGetFuture<String> _sessionPack = ValueGetFuture.loading('loading');
  ValueGetFuture<String> _responsePack = ValueGetFuture.loading('loading');

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addObserver(this);
    initPlatformState();
  }
 
  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    if (state == AppLifecycleState.paused) {
       AdjustCustomTestAlone.onPause();
    } else if (state == AppLifecycleState.resumed) {
       AdjustCustomTestAlone.onResume();
    }
  }

  @override
  Widget build(BuildContext context) {
    _screenWidth = MediaQuery.of(context).size.width;
    _screenHeight = MediaQuery.of(context).size.height;
     return Scaffold(
      appBar: AppBar(
        title: Text('Results'),
        leading: BackButton(onPressed: () {
          Navigator.pushReplacement(
              context, MaterialPageRoute(builder: (_) => FirstScreen()));
        }),
      ),
      body: SingleChildScrollView(
        child: Padding(
          padding: const EdgeInsets.all(8.0),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: [
              //------------ part 1 ------------
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Expanded(
                    child: TextField(
                      textInputAction: TextInputAction.next,
                      keyboardType: TextInputType.number,
                      controller: _info1Controller,
                      decoration: InputDecoration(
                        labelText: WidgetsDefaultValues.info1,
                        suffixIcon: InkWell(
                          onTap: () {
                            _info1Controller.clear();
                            setState(() {});
                          },
                          child: Icon(Icons.clear),
                        ),
                      ),
                    ),
                  ),
                  SizedBox(
                    width: 30,
                  ),
                  Expanded(
                    child: TextField(
                      textInputAction: TextInputAction.next,
                      keyboardType: TextInputType.number,
                      controller: _info2Controller,
                      decoration: InputDecoration(
                        labelText: WidgetsDefaultValues.info2,
                        suffixIcon: InkWell(
                          onTap: () {
                            _info2Controller.clear();
                            setState(() {});
                          },
                          child: Icon(Icons.clear),
                        ),
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 10.0,
              ),
              //------------ part 2 ------------

              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Expanded(
                    child: TextField(
                      textInputAction: TextInputAction.next,
                      keyboardType: TextInputType.number,
                      controller: _info3Controller,
                      decoration: InputDecoration(
                        labelText: WidgetsDefaultValues.info3,
                        suffixIcon: InkWell(
                          onTap: () {
                            _info3Controller.clear();
                            setState(() {});
                          },
                          child: Icon(Icons.clear),
                        ),
                      ),
                    ),
                  ),
                  SizedBox(
                    width: 30,
                  ),
                  Expanded(
                    child: TextField(
                      textInputAction: TextInputAction.next,
                      keyboardType: TextInputType.number,
                      controller: _info4Controller,
                      decoration: InputDecoration(
                        labelText: WidgetsDefaultValues.info4,
                        suffixIcon: InkWell(
                          onTap: () {
                            _info4Controller.clear();
                            setState(() {});
                          },
                          child: Icon(Icons.clear),
                        ),
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 10.0,
              ),
              //------------ part 3 ------------

              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Expanded(
                    child: TextField(
                      textInputAction: TextInputAction.next,
                      keyboardType: TextInputType.number,
                      controller: _appSecretController,
                      decoration: InputDecoration(
                        labelText: WidgetsDefaultValues.appSecret,
                        suffixIcon: InkWell(
                          onTap: () {
                            _appSecretController.clear();
                            setState(() {});
                          },
                          child: Icon(Icons.clear),
                        ),
                      ),
                    ),
                  ),
                  SizedBox(
                    width: 30,
                  ),
                  Expanded(
                      child: ElevatedButton(
                    onPressed: _onTapActivate,
                    child: Text(_isSecretActive?WidgetsDefaultValues
                        .buttonTextActive:WidgetsDefaultValues
                        .buttonTextNotActive),
                    style: ButtonStyle(
                        backgroundColor: MaterialStateProperty.all(
                            _isSecretActive?WidgetsDefaultValues
                            .colorCorrect:WidgetsDefaultValues.colorError)),
                  )),
                ],
              ),
              //------------ part 4 ------------

              SizedBox(
                height: 15,
              ),
              Container(
                  height: _screenHeight / 2,
                  child: Card(
                    elevation: 6,
                    color: Colors.grey.shade300,
                    child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.stretch,
                        children: [
                          Container(
                              padding: EdgeInsets.only(top: 5, bottom: 2),
                              decoration: BoxDecoration(
                                  color: Colors.green,
                                  borderRadius: BorderRadius.only(
                                      topLeft: Radius.circular(6),
                                      topRight: Radius.circular(6))),
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.center,
                                children: [
                                  Text(WidgetsDefaultValues.topInstall),
                                  SizedBox(width: 30.0),
                                  Text(
                                    WidgetsDefaultValues.topStatus +
                                        ' '
                                            '${statusFromValueFuture(_installPack)}',
                                    style:
                                        TextStyle(fontWeight: FontWeight.bold),
                                  ),
                                ],
                              )),
                          Expanded(
                            child: Center(
                              child: Text(_installPack.check()
                                  ? _installPack.data
                                  : _installPack.message),
                            ),
                          ),
                          ElevatedButton(
                            onPressed: _onInstallSendTap,
                            child: Text(WidgetsDefaultValues.send),
                            style: ButtonStyle(
                                backgroundColor: MaterialStateProperty.all(
                                    Colors.grey.shade800)),
                          ),
                        ]),
                  )),
              //------------ part 5 ------------

              SizedBox(
                height: 20,
              ),
              Container(
                  height: _screenHeight / 2,
                  child: Card(
                    elevation: 6,
                    color: Colors.pink.shade200,
                    child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.stretch,
                        children: [
                          Container(
                              padding: EdgeInsets.only(top: 5, bottom: 2),
                              decoration: BoxDecoration(
                                  color: Colors.green,
                                  borderRadius: BorderRadius.only(
                                      topLeft: Radius.circular(6),
                                      topRight: Radius.circular(6))),
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.center,
                                children: [
                                  Text(WidgetsDefaultValues.topSession),
                                  SizedBox(width: 30.0),
                                  Text(
                                    WidgetsDefaultValues.topStatus +
                                        ' '
                                            '${statusFromValueFuture(_sessionPack)}',
                                    style:
                                        TextStyle(fontWeight: FontWeight.bold),
                                  ),
                                ],
                              )),
                          Expanded(
                            child: Center(
                              child: Text(_sessionPack.check()
                                  ? _sessionPack.data
                                  : _sessionPack.message),
                            ),
                          ),
                          ElevatedButton(
                              onPressed: () {
                                AdjustCustomTestAlone.sendSession(_createConfig());
                              },
                              child: Text(WidgetsDefaultValues.send),
                              style: ButtonStyle(
                                  backgroundColor: MaterialStateProperty.all(
                                      Colors.pink.shade800))),
                        ]),
                  )),
              //------------ part 6 ------------

              SizedBox(
                height: 20,
              ),
              Container(
                  height: _screenHeight / 1.5,
                  child: Card(
                    elevation: 6,
                    color: Colors.teal.shade200,
                    child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.stretch,
                        children: [
                          Container(
                              padding: EdgeInsets.only(top: 5, bottom: 2),
                              decoration: BoxDecoration(
                                  color: Colors.green,
                                  borderRadius: BorderRadius.only(
                                      topLeft: Radius.circular(6),
                                      topRight: Radius.circular(6))),
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.center,
                                children: [
                                  Text(WidgetsDefaultValues.topEvent),
                                  SizedBox(width: 30.0),
                                  Text(
                                    WidgetsDefaultValues.topStatus +
                                        ' '
                                            '${statusFromValueFuture(_eventPack)}',
                                    style:
                                        TextStyle(fontWeight: FontWeight.bold),
                                  ),
                                ],
                              )),
                          Expanded(
                            child: Center(
                              child: Text(_eventPack.check()
                                  ? _eventPack.data
                                  : _eventPack.message),
                            ),
                          ),
                          Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              Expanded(
                                child: TextField(
                                  textInputAction: TextInputAction.next,
                                  keyboardType: TextInputType.text,
                                  controller: _eventTokenController,
                                  decoration: InputDecoration(
                                    labelText: WidgetsDefaultValues.eventToken,
                                    suffixIcon: InkWell(
                                      onTap: () {
                                        _eventTokenController.clear();
                                        setState(() {});
                                      },
                                      child: Icon(Icons.clear),
                                    ),
                                  ),
                                ),
                              ),
                              SizedBox(
                                width: 30,
                              ),
                              Expanded(
                                child: TextField(
                                  textInputAction: TextInputAction.done,
                                  keyboardType: TextInputType.text,
                                  controller: _eventValueController,
                                  decoration: InputDecoration(
                                    labelText: WidgetsDefaultValues.eventValue,
                                    suffixIcon: InkWell(
                                      onTap: () {
                                        _eventValueController.clear();
                                        setState(() {});
                                      },
                                      child: Icon(Icons.clear),
                                    ),
                                  ),
                                ),
                              ),
                            ],
                          ),
                          Padding(
                            padding: const EdgeInsets.all(15.0),
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.spaceBetween,
                              children: [
                                Checkbox(
                                    value: _isRevActive,
                                    onChanged: (v) {
                                      setState(() {
                                        _isRevActive = !_isRevActive;
                                      });
                                    }),
                                SizedBox(
                                  width: 30,
                                ),
                                Expanded(
                                  child: TextField(
                                    enabled: _isRevActive,
                                    textInputAction: TextInputAction.next,
                                    keyboardType: TextInputType.number,
                                    controller: _eventRevController,
                                    decoration: InputDecoration(
                                      labelText:
                                          WidgetsDefaultValues.eventRevenue,
                                      suffixIcon: InkWell(
                                        onTap: () {
                                          _eventRevController.clear();
                                          setState(() {});
                                        },
                                        child: Icon(Icons.clear),
                                      ),
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          ),
                          ElevatedButton(
                            onPressed: _onTrackEvent,
                            child: Text(WidgetsDefaultValues.send),
                            style: ButtonStyle(
                                backgroundColor: MaterialStateProperty.all(
                                    Colors.teal.shade800)),
                          ),
                        ]),
                  )),
              SizedBox(
                height: 15,
              ),
              _eventTokenListPack.check()
                  ? ListView.builder(
                shrinkWrap: true,
                itemCount: _eventTokenListPack.data.length,
                itemBuilder: (context, index) {
                  return ListTile(
                    title: Text('[${_eventTokenListPack.data[index]}] '
                        '[${_eventValueListPack.data[index]}] '
                        '[${_eventRevListPack.data[index]}]'),
                    onTap: () {
                      _eventTokenController.text = _eventTokenListPack.data[index];
                      _eventValueController.text = _eventValueListPack.data[index];
                      _eventRevController.text = _eventRevListPack.data[index];
                      setState(() {});
                    },
                    leading: InkWell(
                        child: Icon(Icons.clear),
                        onTap: () {
                          _eventTokenListPack.data.removeAt(index);
                          _eventValueListPack.data.removeAt(index);
                          _eventRevListPack.data.removeAt(index);
                          setState(() {});
                        }),
                  );
                },
              )
                  : Container(width: 0, height: 0),

              SizedBox(
                height: 30,
              ),
              ElevatedButton(
                onPressed: () {},
                child: Padding(
                  padding: const EdgeInsets.symmetric(vertical: 20),
                  child: Text(WidgetsDefaultValues.location),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }



   initPlatformState() async {
    _readEventsFromShp();
    AdjustCustomTestAlone.start(_createConfig());

  }

  @override
  void dispose() {
    super.dispose();
    WidgetsBinding.instance.removeObserver(this);
  }

  _onTrackEvent() {
    _addEventToShp();
    _eventPack = ValueGetFuture.loading('Tracking event');
    AdjustEvent event = AdjustEvent(
      _eventTokenController.text,
    );
    // event.eventValue = _eventValueController.text;

    if (_isRevActive) {
      event.setRevenue(num.parse(_eventRevController.text), 'IRR');
    }
    AdjustCustomTestAlone.trackEvent(event);
  }

  ValueGetFuture<int> _info1Pack=ValueGetFuture.loading('');
  ValueGetFuture<int> _info2Pack=ValueGetFuture.loading('');
  ValueGetFuture<int> _info3Pack=ValueGetFuture.loading('');
  ValueGetFuture<int> _info4Pack=ValueGetFuture.loading('');
  ValueGetFuture<int> _appSecretPack=ValueGetFuture.loading('');
  _onTapActivate(){
    if(_info1Controller.text.isNotEmpty && _info2Controller.text.isNotEmpty
        && _info3Controller.text.isNotEmpty && _info4Controller.text
        .isNotEmpty && _appSecretController.text.isNotEmpty){
      _info1Pack = ValueGetFuture.completed(int.parse(_info1Controller.text));
      _info2Pack = ValueGetFuture.completed(int.parse(_info2Controller.text));
      _info3Pack = ValueGetFuture.completed(int.parse(_info3Controller.text));
      _info4Pack = ValueGetFuture.completed(int.parse(_info4Controller.text));
      _appSecretPack = ValueGetFuture.completed(int.parse
        (_appSecretController.text));
      _isSecretActive=true;


    }else{
      _isSecretActive=false;
      Fluttertoast.showToast(msg:WidgetsDefaultValues.errorCheckInputs,
          gravity: ToastGravity.CENTER);

    }
    setState(() {});
  }

  AdjustConfig _createConfig() {
    AdjustConfig _sdkConfig = AdjustConfig(
        widget.baseUrl, widget.appToken, AdjustEnvironment.sandbox);

    if(_info1Pack.check() && _info2Pack.check() && _info3Pack.check() &&
        _info4Pack.check() && _appSecretPack.check()){
      _sdkConfig.setAppSecret(_appSecretPack.data, _info1Pack.data, _info2Pack.data, _info3Pack.data,
          _info4Pack.data);
    }

    _sdkConfig.logLevel = AdjustLogLevel.verbose;


    _sdkConfig.sdkCallbackSucceed = (SdkClickSucceed success) {
      print(success);
      _installPack = ValueGetFuture.completed(success.sdkClick,
          statusCode: success.statusCode);
      setState(() {});
    };
    _sdkConfig.sdkCallbackFailure = (SdkClickFailure failure) {
      print(failure);

      _installPack = ValueGetFuture.error(failure.sdkClick,
          statusCode: failure.statusCode);
      setState(() {});
    };
    _sdkConfig.eventSuccessCallback = (AdjustEventSuccess success) {
      print(success);

      _eventPack = ValueGetFuture.completed(success.jsonResponse);
      setState(() {});
    };
    _sdkConfig.eventFailureCallback = (AdjustEventFailure failure) {
      print(failure);

      _eventPack = ValueGetFuture.error(failure.jsonResponse);
      setState(() {});
    };
    _sdkConfig.sessionSuccessCallback = (AdjustSessionSuccess success) {
      print(success);

      _sessionPack = ValueGetFuture.completed(success.jsonResponse);
      setState(() {});
    };
    _sdkConfig.sessionFailureCallback = (AdjustSessionFailure failure) {
      print(failure);
      _sessionPack = ValueGetFuture.error(failure.jsonResponse);
      setState(() {});
    };
    // Add session callback parameters.
    // AdjustCustomTestAlone.addSessionCallbackParameter('scp_foo_1', 'scp_bar');
    // AdjustCustomTestAlone.addSessionCallbackParameter('scp_foo_2', 'scp_value');
    //
    // // Add session Partner parameters.
    // AdjustCustomTestAlone.addSessionPartnerParameter('spp_foo_1', 'spp_bar');
    // AdjustCustomTestAlone.addSessionPartnerParameter('spp_foo_2', 'spp_value');
    //
    // // Remove session callback parameters.
    // AdjustCustomTestAlone.removeSessionCallbackParameter('scp_foo_1');
    // AdjustCustomTestAlone.removeSessionPartnerParameter('spp_foo_1');
    //
    // // Clear all session callback parameters.
    // AdjustCustomTestAlone.resetSessionCallbackParameters();

    // Clear all session partner parameters.
    // AdjustCustomTestAlone.resetSessionPartnerParameters();
    return _sdkConfig;
  }

  ValueGetFuture<List<String>> _eventTokenListPack;
  ValueGetFuture<List<String>> _eventValueListPack;

  ValueGetFuture<List<String>> _eventRevListPack;

  void _onInstallSendTap() {
    AdjustCustomTestAlone.sendInstall(_createConfig());
  }

  Future<void> _readEventsFromShp() async {
    _eventTokenListPack = ValueGetFuture.loading('');
    _eventValueListPack = ValueGetFuture.loading('');
    _eventRevListPack = ValueGetFuture.loading('');

    List<String> tokens =
        await ShpHelper.readStringList(StorageKeys.event_token_list_key);
    List<String> values =
        await ShpHelper.readStringList(StorageKeys.event_value_list_key);
    List<String> revs =
        await ShpHelper.readStringList(StorageKeys.event_rev_list_key);

    if (tokens.isNotEmpty && values.isNotEmpty && revs.isNotEmpty) {
      _eventRevListPack = ValueGetFuture.completed(tokens);
      _eventRevListPack = ValueGetFuture.completed(values);
      _eventRevListPack = ValueGetFuture.completed(revs);
    } else {
      _eventRevListPack = ValueGetFuture.error('');
      _eventRevListPack = ValueGetFuture.error('');
      _eventRevListPack = ValueGetFuture.error('');
    }
    setState(() {});
  }

  Future<void> _addEventToShp() async {
    List<String> tokens =
        _eventTokenListPack.data;
    List<String> values =
        _eventValueListPack.data;
    List<String> revs =
        _eventRevListPack.data;

    bool ok = false;
    if(tokens!=null) {
      if (tokens.contains(_eventTokenController.text)) {
        int i = tokens.indexOf(_eventTokenController.text);
        if (values[i] == _eventValueController.text &&
            revs[i] == _eventRevController.text) {
          ok = false;
        } else {
          ok = true;
        }
      } else {
        ok = true;
      }
    }else{
      ok = true;
    }

    if (ok) {
      tokens.add(_eventTokenController.text);
      values.add(_eventValueController.text.isNotEmpty
          ? _eventValueController.text
          : '-1');
      revs.add(_eventRevController.text.isNotEmpty
          ? _eventRevController.text
          : '-1');
      await ShpHelper.writeStringList(StorageKeys.event_token_list_key, tokens);
      await ShpHelper.writeStringList(StorageKeys.event_value_list_key, values);
      await ShpHelper.writeStringList(StorageKeys.event_rev_list_key, revs);
    }
    setState(() {});
  }
}
