import 'package:flutter/material.dart';

class StorageKeys {
  static const String app_tokens_list_key = 'app_tokens_list_key';
  static const String base_url_list_key = 'base_url_list_key';
  static const String info1_key = 'info1_key';
  static const String info2_key = 'info2_key';
  static const String info3_key = 'info3_key';
  static const String info4_key = 'info4_key';
  static const String secret_id_key = 'secret_id_key';
  static const String event_token_list_key = 'event_token_list_key';
  static const String event_value_list_key = 'event_value_list_key';
  static const String event_rev_list_key = 'event_rev_list_key';
}

class WidgetsDefaultValues {
  static const String baseUrl = 'Base Url';
  static const String appToken = 'App Token';
  static const String pageTitle = 'Adjust Dev Test';

  static const String info1 = 'Info 1';
  static const String info2 = 'Info 2';
  static const String info3 = 'Info 3';
  static const String info4 = 'Info 4';
  static const String appSecret = 'App Secret ID';
  static const String buttonTextNotActive = 'Not Activated';
  static const String buttonTextActive = 'Activated';

  static const String topInstall = 'Install';
  static const String topStatus = 'Status';
  static const String topAttribution = 'Attribution';
  static const String topSession = 'Session';
  static const String topEvent = 'Event';

  static const String eventToken = 'Event Token';
  static const String eventValue = 'Event Value';
  static const String eventRevenue = 'Event Revenue';

  static const String location = 'Location';

  static const String start = 'Start';
  static const String reset = 'Reset';
  static const String gpsAdId = 'GPS Ad ID';

  static const String resume = 'Start/Resume SDK';
  static const String pause = 'Pause/Stop SDK';

  static const String loading = 'Loading...';
  static const String error = 'Error!';

  static const String send = 'Send';

  static const String eventData = 'Event Data';
  static const String installData = 'Install Data';
  static const String attributionData = 'Attribution Data';
  static const String sessionData = "Session Data";

  static const String errorCheckInputs = 'Make sure to enter input values '
      'correctly!';

  static const String attributionLoading = 'Attribution data is loading!';

  static const Color colorStatus2nn = Colors.green;
  static const Color colorStatus4nn = Colors.redAccent;
  static const Color colorStatus5nn = Colors.white;

  static final Color colorEnable = Colors.blue;
  static final Color colorDisable = Colors.grey.shade400;
  static final Color colorError = Colors.red.shade800;
  static final Color colorCorrect = Colors.green.shade800;
}

class ValueGetFuture<T> {
  Status status;
  T data;
  String message;
  int statusCode;

  ValueGetFuture.loading(this.message) : status = Status.LOADING;

  ValueGetFuture.completed(this.data, {this.statusCode})
      : status = Status.COMPLETED;

  ValueGetFuture.error(this.message, {this.statusCode}) : status = Status.ERROR;

  @override
  String toString() {
    return "Status : $status \n Message : $message \n Data : $data";
  }

  bool check() {
    bool res = false;
    if (status != null && status == Status.COMPLETED) {
      res = true;
    }
    return res;
  }
}

enum Status { LOADING, COMPLETED, ERROR }

bool isValueStringListOk(dynamic data) {
  bool res = false;
  if (data != null) {
    if (data is String)
      res = data.isNotEmpty;
    else if (data is List) res = data.length > 0;
  }
  return res;
}

enum SDKState { Resumed, Paused }

String mapPrettifyMap(Map<String, dynamic> map) {
  String res = '';
  res = res + '{\n';
  List<String> keys = map.keys;
  List values = map.values;
  for (int i = 0; i < keys.length; i++) {
    if (values[i] is Map) {
      res = res + mapPrettifyMap(values[i]);
    } else {
      res = res + '${keys[i]} : ${values[i]}\n';
    }
  }

  res = res + '\n}';
}

String mapPrettifyStr(String input) {
  String res = '';
  res = input.replaceAll('{', '{\n');
  res = res.replaceAll('}', '\n}');
  res = res.replaceAll(',', ',\n');

  return res;
}

String statusFromValueFuture(ValueGetFuture v) {
  if (v.status == Status.LOADING) {
    return 'Loading...';
  } else if (v.status == Status.COMPLETED) {
    return '${v.statusCode}';
  } else if (v.status == Status.ERROR) {
    return '${v.statusCode}';
  }
}

